/*
**  $Id: DB_host.h,v 1.1 1999/04/27 20:21:28 jms Exp $
**  GXSNMP -- An snmp management application
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc.,  59 Temple Place - Suite 330, Cambridge, MA 02139, USA.
**
**  Functions for dealing with DB_host objects.
*/

#ifndef __DB_HOST_H__
#define __DB_HOST_H__

#include "gtkdatabase.h"

/*****************************************************************************
**
**  Standard widget macros
**
*****************************************************************************/

#define DB_TYPE_HOST (DB_host_get_type())

#define DB_HOST(obj) \
        GTK_CHECK_CAST ((obj), DB_TYPE_HOST, DB_host)

#define DB_HOST_CLASS(klass) \
        GTK_CHECK_CLASS_CAST (klass, DB_TYPE_HOST, \
                              DB_hostClass)

#define DB_IS_HOST(obj) \
        GTK_CHECK_TYPE ((obj), DB_TYPE_HOST)

#define DB_IS_HOST_CLASS(klass) \
        GTK_CHECK_CLASS_TYPE ((klass), DB_TYPE_HOST)


/*****************************************************************************
**
**  Control blocks for the widget class and for widget instances
**
*****************************************************************************/

typedef struct _DB_hostClass        DB_hostClass;
typedef struct _DB_host             DB_host;

struct _DB_hostClass
{
  GtkDatabaseItemClass    database_item_class;  /* Base class */
};

struct _DB_host
{
  GtkDatabaseItem database_item;	/* Base object */
  guint           rowid;                /* Unique row identifier */
  gchar         * created;              /* Row creation date/time */
  gchar         * modified;             /* Last row modification date/time */
  gchar         * dns_name;             /* Preferred DNS name of host */
  gchar         * name;                 /* Name of host to display on maps */
  gchar         * description;          /* User-provided description of host */
  gchar         * contact;              /* User-provided contact information */
  gpointer        g_sqldb_private;      /* For use by the g_sqldb library */

  GList         * DB_interfaces;        /* GList of all interfaces assigned to
                                           this host */
  GList         * DB_graphs;            /* GList of graphical representations
                                           of this host */

  gpointer        application;          /* Pointer to application-specific
                                           data for the host.  Currently
                                           unused. */
};

/******************************************************************************
**
**  The public API for manipulating DB_host objects
**
******************************************************************************/

DB_host  * DB_host_new  	(void);
void       host_destroy 	(DB_host * dbh);
void       host_add     	(DB_host * dbh);
void       host_update  	(DB_host * dbh);
void       host_delete  	(DB_host * dbh);
GList	 * host_list		(void);
DB_host  * host_find_by_rowid 	(guint     rowid);
DB_host  * host_find_by_name    (gchar   * name);

#endif

/* EOF */
