/*
 *  $Id: network.c,v 1.4 1999/04/23 23:41:59 jms Exp $
 *
 *  GXSNMP -- An snmp management application
 *  Copyright (c) 1998 Gregory McLean
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc.,  59 Temple Place - Suite 330, Cambridge, MA 02139, USA.
 */

/* 
 * net.c containes utility functions related to the creation and destruction
 * of DB_network items, and functions to manage the DB_network database.
 */
#include <gnome.h>
#include "g_sqldb.h"
#include "tables.h"
#include "db_interface.h"
#include "interface.h"
#include "graph.h"

DB_network *
network_create (void)
{
  DB_network *dbn;
  
  dbn           = g_new0 (DB_network, 1);
  dbn->created  = g_strdup (_("(not in database)"));
  dbn->modified = g_strdup (dbn->created);
  dbn->DB_graphs= NULL;
  return dbn;

}

void 
network_destroy (DB_network *dbn)
{
  g_assert (dbn->rowid == 0);
  
  if (dbn->created)        g_free (dbn->created);
  if (dbn->modified)       g_free (dbn->modified);
  if (dbn->name)           g_free (dbn->name);
  if (dbn->address)        g_free (dbn->address);
  if (dbn->netmask)        g_free (dbn->netmask);
  if (dbn->description)    g_free (dbn->description);
  if (dbn->contact)        g_free (dbn->contact);

  g_free (dbn);

}

void
network_add (DB_network *dbn)
{
  g_assert (dbn->rowid == 0);
  dbn->rowid = g_sqldb_highest_rowid (network_sqldb, "_rowid") + 1;
  if (dbn->created) g_free (dbn->created);
  dbn->created = db_timestamp ();
  if (dbn->modified) g_free (dbn->modified);
  dbn->modified = g_strdup (dbn->created);

  g_sqldb_row_add (network_sqldb, dbn);

}


void 
network_update (DB_network *dbn)
{
  g_assert (dbn->rowid != 0);
  if (dbn->modified) g_free (dbn->modified);
  dbn->modified = db_timestamp ();
  g_sqldb_row_update (network_sqldb, dbn);

}

void
network_delete (DB_network *dbn)
{
  GList   *gl;
  while ((gl = dbn->DB_graphs))		/* Assignment intended */
    {
      graph_delete ((DB_graph *)gl->data);
      graph_destroy ((DB_graph *)gl->data);
    }
  g_assert (g_sqldb_row_delete (network_sqldb, dbn) == TRUE);
  dbn->rowid = 0;

}

GList *
network_list (void)
{
  return g_sqldb_table_list (network_sqldb);
}

DB_network *
network_find_by_rowid (guint rowid)
{
  return g_sqldb_row_find (network_sqldb, "_rowid", &rowid);
}

/*
**  Subroutine used to find all network segments assigned to a subnet.
**  In the case of switched ethernet, there may be many segments with
**  the same IP range.  This subroutine returns a GList of them.
*/

GList *
network_find_by_address (gchar * address, gchar * netmask)
{
  GList      * gl;
  GList      * result = NULL;
  DB_network * dbn;
  gboolean     matched = FALSE;

  if (!address) return NULL;

  gl = g_sqldb_table_list (network_sqldb);
  while (gl)
    {
      matched = FALSE;
      dbn = gl->data;
      if (dbn->address)
        if (!strcmp (address, dbn->address))
	  {
	    if (!netmask && !dbn->netmask)
	      matched = TRUE;
	    if (netmask && dbn->netmask)
	      if (!strcmp (netmask, dbn->netmask))
	        matched = TRUE;
	    if (matched)
	      result = g_list_append (result, dbn);
	  }
      gl = gl->next;
    }
  return result;
}

/* EOF */

