/*
 *  $Id: snmp.c,v 1.6 1999/04/03 16:17:08 gregm Exp $
 *
 *  GXSNMP -- An snmp management application
 *  Copyright (c) 1998 Gregory McLean
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc.,  59 Temple Place - Suite 330, Cambridge, MA 02139, USA.
 */

/* 
 * snmp.c containes utility functions related to the creation and destruction
 * of DB_snmp items, and functions to manage the DB_snmp database.
 */
#include <gnome.h>
#include "g_sqldb.h"
#include "tables.h"
#include "snmp.h"

DB_snmp *
snmp_create (void)
{
  DB_snmp *dbs;
  
  dbs           = g_new0 (DB_snmp, 1);
  dbs->created  = g_strdup (_("(not in database)"));
  dbs->modified = g_strdup (dbs->created);

  return dbs;

}

void 
snmp_destroy (DB_snmp *dbs)
{
  g_assert (dbs->rowid == 0);
  
  if (dbs->created)        g_free (dbs->created);
  if (dbs->modified)       g_free (dbs->modified);
  if (dbs->name)           g_free (dbs->name);
  if (dbs->read_c)         g_free (dbs->read_c);
  if (dbs->write_c)        g_free (dbs->write_c);

  g_free (dbs);

}

void
snmp_add (DB_snmp *dbs)
{
  g_assert (dbs->rowid == 0);
  dbs->rowid = g_sqldb_highest_rowid (snmp_sqldb, "_rowid") + 1;
  if (dbs->created) g_free (dbs->created);
  dbs->created = db_timestamp ();
  if (dbs->modified) g_free (dbs->modified);
  dbs->modified = g_strdup (dbs->created);

  g_sqldb_row_add (snmp_sqldb, dbs);

}


void 
snmp_update (DB_snmp *dbs)
{
  g_assert (dbs->rowid != 0);
  if (dbs->modified) g_free (dbs->modified);
  dbs->modified = db_timestamp ();
  g_sqldb_row_update (snmp_sqldb, dbs);

}

void
snmp_delete (DB_snmp *dbs)
{
  g_assert (g_sqldb_row_delete (snmp_sqldb, dbs) == TRUE);
  dbs->rowid = 0;
}

GList *
snmp_list (void)
{
  return g_sqldb_table_list (snmp_sqldb);
}

DB_snmp *
snmp_find_by_rowid (guint rowid)
{
  return g_sqldb_row_find (snmp_sqldb, "_rowid", &rowid);
}

/* EOF */



