
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <glib.h>
#include "g_snmp.h"
#include "mib.h"
#include "parse.h"

#include "snmp_lib.h"
#include "ping.h"


void readsline(gpointer data, gpointer user_data);
void checkout_host(u_long address);
		
void
cb_register(guint socket, void (*receiveMessage)())
{
    /*gdk_input_add(socket, GDK_INPUT_READ, receiveMessage, NULL);*/
    turtle_input_read_add(socket, receiveMessage);
}
void
snmpinit()
{
	if (!g_snmp_init(FALSE))
		g_error("Initialisation of SNMP library failed.");
/*	gtk_timeout_add (1000, g_snmp_timeout_cb, NULL);*/
}

void 
readscol(gpointer data, gpointer user_data)
{
	g_print("<> %s", (char*)data);
}


void
gotline_callback(host_snmp *host, GSList *results)
{
	g_print("line callback: ");
	g_slist_foreach((GSList*)results, readscol, NULL);
	g_print("\n");
}

void
readsline(gpointer data, gpointer user_data)
{
	g_slist_foreach((GSList*)data, readscol, NULL);
	g_print("\n");
}

/*
 * This function is called once we have sent a ping to a device and it has
 * replied back.  Now we want to SNMP it and find out more!
 */
void
got_ping(u_long address, time_t sent_time, gpointer user_data)
{
    struct in_addr in;
    in.s_addr = address;
    g_print("Received ping from %s\n", inet_ntoa(in));
    checkout_host(address);
}
int
main(int argc, char *argv[])
{
    u_long seed;
    GMainLoop *loop;
    
    seed = inet_addr("127.0.0.1");

/*	gtk_init(&argc, &argv);*/
	snmpinit();
	
	init_mib();

        init_ping();
        sendping(ntohl(INADDR_LOOPBACK), got_ping, NULL, NULL);

/*	gtk_main();*/
    /*
     * Now comes the scary glib loop stuff 
     */
    loop = g_main_new(TRUE);
    while(g_main_is_running(loop))
    {
        g_main_run(loop);
    }
    g_main_destroy(loop);
    return 0;
}
	
void
ifline_callback(host_snmp *host, GSList *results, gpointer user_data)
{
	GSList *adsl, *nmsl;
	char *address, *netmask;
	struct in_addr addr, netm, neta;

	adsl = g_slist_nth(results, 0);
	nmsl = g_slist_nth(results, 1);
	address = (char*)adsl->data;
	netmask = (char*)nmsl->data;

	inet_aton(address, &addr);
	inet_aton(netmask, &netm);
	
	neta.s_addr = addr.s_addr & netm.s_addr;
	g_print("New network %s with netmask %s\n", inet_ntoa(neta), netmask);
}

void
atline_callback(host_snmp *host, GSList *results, gpointer user_data)
{
    GSList *adsl;
	unsigned char *address;

	adsl = g_slist_nth(results, 0);

        address = (unsigned char*)adsl->data;

	g_print("New node found %s\n", address );
        sendping(inet_addr(address), got_ping, NULL, NULL);
/*        checkout_host(inet_addr(address));*/
}

void
checkout_host(u_long address)
{

    GSList *ifobjs, *atobjs;

    ifobjs = NULL;
    atobjs = NULL;

    /* First we find all interfaces, which tells us what networks the
     * device is connected to
     */

    ifobjs = g_slist_append(ifobjs, "ip.ipAddrTable.ipAddrEntry.ipAdEntAddr");
    ifobjs = g_slist_append(ifobjs, "ip.ipAddrTable.ipAddrEntry.ipAdEntNetMask");
    get_snmp_table(address, ifobjs, ifline_callback, NULL);

    atobjs = g_slist_append(atobjs, "at.atTable.atEntry.atNetAddress");
    atobjs = g_slist_append(atobjs, "at.atTable.atEntry.atPhysAddress");
    atobjs = g_slist_append(atobjs, "at.atTable.atEntry.atIfIndex");
    get_snmp_table(address, atobjs, atline_callback, NULL);
}

