#include <netinet/in.h>
#include <arpa/inet.h>
#include <gtk/gtk.h>
#include "g_snmp.h"
#include "mib.h"
#include "parse.h"
#include "snmp_lib.h"
#include "host_list.h"

static void time_callback(host_snmp *host, void *magic);
static gboolean done_callback(host_snmp *host, void *magic, SNMP_PDU *spdu, GSList *objs);

static void
add_line_to_request(gpointer data, gpointer user_data)
{
	GsnmpLineRequest *req = (GsnmpLineRequest*)user_data;
	char *what = (char*)data;
	struct _SNMP_OBJECT *obj;
	oidentry *var;

	obj = g_malloc(sizeof(struct _SNMP_OBJECT));
	
	obj->type       = SNMP_NULL;
	obj->syntax_len = 0;
	obj->id_len     = SNMP_SIZE_OBJECTID;
	obj->id         = g_malloc(SNMP_SIZE_OBJECTID);

	if (!read_objid(what, obj->id, &obj->id_len))
	{
		g_warning("Invalid snmp variable \"%s\"!", what);
		g_free(obj);
		return;
	}
	var = g_malloc(sizeof(oidentry));
	var->id_len = obj->id_len;
	g_memmove(var->id, obj->id, obj->id_len * sizeof(gulong));
	var->oid_name = g_strdup(what);
	req->objs = g_slist_append(req->objs, obj);
	req->entry = g_slist_append(req->entry, var);
}

void
get_snmp_table(u_long address, GSList *obslist, GsnmpLineCB line_cb, gpointer user_data)
{
    GsnmpLineRequest *req = NULL;
    host_snmp *host = NULL;
       
    host = (host_snmp*)malloc(sizeof(host_snmp));
    g_print("D: getting info for %s\n", inet_ntoa(*(struct in_addr*)&address));

    host->domain = AF_INET;
    host->rcomm = "public";
    host->wcomm = "public";
    host->retries = 0;
    host->name = g_strdup(inet_ntoa(*(struct in_addr*)&address));
    host->status = 0;
    host->port = 0;
    host->timeout = 0;
    host->version = SNMP_V1;
    host->done_callback = done_callback;
    host->time_callback = time_callback;
    host->magic = NULL;

    req = g_malloc(sizeof(GsnmpLineRequest));
    req->line_cb = line_cb;
    req->objs = NULL;
    req->entry = NULL;
    req->user_data = user_data;
    g_slist_foreach(obslist, add_line_to_request, req);

    host->magic = req;
    req->request = g_async_getnext(host, req->objs);
}


static gboolean
done_callback(host_snmp *host, void *magic, SNMP_PDU *spdu, GSList *objs)
{
  char                buf[1024];              /* Should be big enough? */
  GSList              *entry;
  GSList              *myentry;
  GSList              *myobj;
  GSList              *nobjs;
  oidentry            *var;
  struct _SNMP_OBJECT *obj;
  struct _SNMP_OBJECT *nobj;
  int                  i, j;
  static GHashTable   *if_desc = NULL;
  struct tree         *snmptree;
  GSList *results;
  GsnmpLineRequest *req;

#ifdef MEM_DEBUG
  DMC ();
#endif
  /*g_print("D: done_callback %s\n",host->name);*/
  req = (GsnmpLineRequest*)magic;
  if (req->request)
    req->request = NULL;
  else {
      g_warning("done_callback(): Received SNMP reply, no request, huh?\n");
      return TRUE;
  }

  entry = req->entry;
  results = NULL;
  if (!if_desc)
      if_desc = g_hash_table_new (g_int_hash, g_int_equal);

  if (spdu->request.error_status)
    {
      g_slist_free(entry);
      return TRUE;
    }

  nobjs = NULL;
  myobj = objs;
  i     = 0;
  j     = 0;
  while(myobj)
    {
      obj = (struct _SNMP_OBJECT *) myobj->data;
      g_snmp_printf(buf, sizeof(buf), obj);
      results = g_slist_append(results, g_strdup(buf));
      myentry = entry;
      i = 0;
      while (myentry)
        {
          var = (oidentry *) myentry->data;
          if (!memcmp(obj->id, var->id, var->id_len * sizeof(gulong)))
            {
	      snmptree = get_mib_tree (var->id, var->id_len);
              nobj = g_malloc(sizeof(struct _SNMP_OBJECT));
              nobj->type       = SNMP_NULL;
	      nobj->syntax_len = 0;
              nobj->id_len     = obj->id_len;
              nobj->id         = g_malloc(obj->id_len * sizeof(gulong));
              g_memmove(nobj->id, obj->id, obj->id_len * sizeof(gulong));
              nobjs  = g_slist_append(nobjs, nobj);
              j++;
            }
          i++;
          myentry = g_slist_next (myentry);
        }
      myobj = myobj->next;
    }

  if (i != j)
    {
      g_slist_free(nobjs);
      nobjs = NULL;
    }
    if (nobjs)
    {
        /* This calls the user-defined function that is called on each row */
        req->line_cb(host, results, req->user_data);

	host->done_callback = done_callback;
	host->time_callback = time_callback;
	host->magic = req;
        req->objs = nobjs;
	req->request = g_async_getnext(host, req->objs);
    }
    else 
    {
        g_slist_free(entry);
        req->request = NULL;
        g_print("SNMP request finished for %s \n", host->name);
        free(host); 
    }
    return TRUE;
}

static void
time_callback(host_snmp *host, void *magic)
{
    GsnmpLineRequest *req;
    req = (GsnmpLineRequest*)magic;
    if (req->request)
        req->request = NULL;
    else {
        g_warning("Got a SNMP timeout with no request being made.\n");
        return;
    }
    g_warning("Timeout getting SNMP data from %s\n", host->name);
}

gboolean    
turtle_io_invoke(GIOChannel *source, GIOCondition condition, gpointer data)
{
    void (*func)(void);

    func = data;
    func();
    return TRUE;
}

void
turtle_input_read_add(int socket, void(*receiveMessage)())
{
    guint result;
    GIOChannel *channel;
    GIOCondition cond;
        
    cond = (G_IO_IN | G_IO_PRI); /* GDK_INPUT_READ */
    channel = g_io_channel_unix_new(socket);
    result = g_io_add_watch_full(channel, G_PRIORITY_DEFAULT, cond,
            turtle_io_invoke, receiveMessage, NULL);
    g_io_channel_unref(channel);
}
