/*
**  GXSNMP -- An snmp management application
**  Copyright (C) 1998 Jochen Friedrich
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc.,  59 Temple Place - Suite 330, Cambridge, MA 02139, USA.
**
**  plugin-mtr.c -- A sample plugin, it just adds the MTR command to the host
**                  menu for testing.
**
**  MTR stands for "Matt's Traceroute"
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib.h>
#include "plugins.h"
#include "structs.h"
#include "main.h"

/* There should *really* be a public method of the individual map_item objects 
** to add more general menu items if a user wants this. Some external programs
** might be of general interest, no matter of the sub type. mtr, eg. can be
** useful no matter if the host object is a database server, a printer or
** some sort of router.
*/

#include "gxsnmp_map.h"
#include "gxsnmp_host.h"

/* FIXME: You must remove the static attribute in menu.c to make this work. Else
** you get an undefined symbol if you load this module
*/

extern GtkWidget * host_popup_menu;

extern gxsnmp *app_info;

/******************************************************************************
**
**  Forward references
**
******************************************************************************/

static void menu_callback (GtkWidget * widget, gpointer data);
static void trace_menu_cb (GtkWidget * widget, gpointer data);

/******************************************************************************
**
**  Static data
**
******************************************************************************/

static gchar * menu_location;

/******************************************************************************
**
**  The plugin load routine
**
******************************************************************************/

gint
load_plugin (PluginData * pd)
{
  pd->type = PLUGIN_APPLICATION;
  return 0;
}

/******************************************************************************
**
**  The plugin unload routine
**
******************************************************************************/

void
unload_plugin (PluginData * pd)
{
  g_print ("MTR plugin unloaded\n");
}

/******************************************************************************
**
**  This sucks rocks!!!!
**
**  Apparently, there is no easy way to extend a gnome popup list.
**  It looks like you have to duplicate all the popup functions and hope
**  they don't change, as the connect function is taken from gnome and the
**  marshal function from us... Did they really oversee the need to extend
**  a popup??? I can't believe this!!!
**
******************************************************************************/

static void
popup_connect_func (GnomeUIInfo *uiinfo, gchar *signal_name, GnomeUIBuilderData *uibdata)
{

/* This doesn't seem to get used, at all... */

  g_assert (uibdata->is_interp);

  gtk_object_set_data (GTK_OBJECT (uiinfo->widget),
                       "gnome-popup-menu:toplevel_menushell_key",
                       host_popup_menu);

  gtk_signal_connect_full (GTK_OBJECT (uiinfo->widget), signal_name,
                           NULL,
                           uibdata->relay_func,
                           uiinfo->moreinfo,
                           uibdata->destroy_func,
                           FALSE,
                           FALSE);
}

typedef void (* ActivateFunc) (GtkObject *object, gpointer data);

static GtkWidget*
get_toplevel(GtkWidget* menuitem)
{
        return gtk_object_get_data (GTK_OBJECT (menuitem),
                                    "gnome-popup-menu:toplevel_menushell_key");
}

static void
popup_marshal_func (GtkObject *object, gpointer data, guint n_args, GtkArg *args)
{
  ActivateFunc func;
  gpointer user_data;

  func = (ActivateFunc) data;
  user_data = gtk_object_get_data (GTK_OBJECT (get_toplevel(GTK_WIDGET (object))),
                                   "gnome_popup_menu_do_popup_user_data");

  gtk_object_set_data (GTK_OBJECT (GTK_WIDGET (object)->parent),
                       "popup_active_item", object);
  if (func)
    (* func) (object, user_data);
}

/******************************************************************************
**
**  The plugin start routine
**
******************************************************************************/

gint
start_plugin (PluginData * pd)
{
  GnomeUIInfo   * menu;
  GtkWidget     * menu_item;
  GnomeUIBuilderData uibdata;

  g_print ("Starting MTR plugin.\n");

  menu_location         = g_strdup ("Plugins/");

  menu                  = g_malloc0 (2 * sizeof(GnomeUIInfo));
  menu->type            = GNOME_APP_UI_ITEM;
  menu->label           = g_strdup ("MTR");
  menu->hint            = NULL;
  menu->moreinfo        = menu_callback;       /* Menu activation cb routine */
  menu->user_data       = NULL;                /* User data for the callback */
  menu->unused_data     = NULL;                /* Reserved for future use */
  menu->pixmap_type     = 0;
  menu->pixmap_info     = NULL;
  menu->accelerator_key = 0;
  (menu + 1)->type      = GNOME_APP_UI_ENDOFINFO;

  menu                  = g_malloc0 (2 * sizeof(GnomeUIInfo));
  menu->type            = GNOME_APP_UI_ITEM;
  menu->label           = g_strdup ("MTR Host");
  menu->hint            = NULL;
  menu->moreinfo        = trace_menu_cb;       /* Menu activation cb routine */
  menu->user_data       = NULL;                /* User data for the callback */
  menu->unused_data     = NULL;                /* Reserved for future use */
  menu->pixmap_type     = GNOME_APP_PIXMAP_STOCK;
  menu->pixmap_info     = GNOME_STOCK_MENU_BLANK;
  menu->accelerator_key = 0;
  (menu + 1)->type      = GNOME_APP_UI_ENDOFINFO;

  uibdata.connect_func = popup_connect_func;
  uibdata.data = NULL;
  uibdata.is_interp = TRUE;
  uibdata.relay_func = popup_marshal_func;
  uibdata.destroy_func = NULL;

  gnome_app_fill_menu_custom(host_popup_menu, menu, &uibdata, NULL, FALSE, 0);

  return 0;
}

/******************************************************************************
**
**  The menu callback routine
**
******************************************************************************/

static void
menu_callback (GtkWidget * widget, gpointer data)
{
  g_print ("MTR plugin menu callback invoked\n");
}

void
host_panel_trace(char *host)
{
  char buffer[1024];

  snprintf(buffer, sizeof(buffer), "mtr %s &", host);
  system(buffer);
}
 
static void           
trace_menu_cb (GtkWidget *widget, gpointer data)
{
  GXsnmp_map_item * item;
  DB_host         * dbh;
  GtkWidget       * dialog_widget;

  g_return_if_fail (GXSNMP_IS_HOST (data));
  item = GXSNMP_MAP_ITEM (data);
  g_return_if_fail (item->DB_graph != NULL);
  g_return_if_fail (item->DB_graph->type == DB_GRAPH_HOST);
  dbh = item->DB_graph->DB_host;
  g_return_if_fail (dbh != NULL);

  host_panel_trace (dbh->dns_name);
}

/* EOF */
